<?php
AGSLayouts::VERSION; // Access control

include_once('site-export/caldera.php');

?>
<div id="ags-layouts-settings">
	<div id="ags-layouts-settings-header">
		<h1><?php esc_html_e('WP Layouts Site Exporter', 'wp-layouts-td');?></h1>
		<div id="ags-layouts-settings-header-links">
			<a id="ags-layouts-settings-header-link-settings" href="admin.php?page=ags-layouts-settings"><?php esc_html_e('Settings', 'wp-layouts-td');?></a>
			<a id="ags-layouts-settings-header-link-support" href="https://support.wplayouts.space/" target="_blank"><?php esc_html_e('Support', 'wp-layouts-td');?></a>
		</div>
	</div>
	<div id="ags-layouts-site-export">
		<form method="post">
			<label>
				<?php esc_html_e('Export Name', 'wp-layouts-td');?>
				<input id="ags-layouts-site-export-name" type="text" value="<?php echo esc_attr( get_bloginfo() );  ?>" required>
			</label>
			
			<label>
				<?php esc_html_e('Export Image (optional; PNG format, recommended size 900x750)', 'wp-layouts-td');?>
				<input id="ags-layouts-site-export-image" type="file">
			</label>
			
			<ul id="ags-layouts-site-export-items">
				<li>
					<label>
						<input type="checkbox" name="exportItems[]" value="content" checked disabled>
						<?php esc_html_e('Site content (includes pages, posts, categories, menus, etc.)', 'wp-layouts-td');?>
					</label>
					
					
				<ul id="ags-layouts-site-export-content-exclude">
					<li>
						<label class="full-size">
							<?php esc_html_e('Exclude post types (comma separated):', 'wp-layouts-td');?>
							<input type="text" name="excludePostTypes" value="">
						</label>
					</li>
					<li>
						<label class="full-size">
							<?php esc_html_e('Exclude taxonomies (comma separated):', 'wp-layouts-td');?>
							<input type="text" name="excludeTaxonomies" value="module_width,scope,layout_type,action-group">
						</label>
						
					</li>
				</ul>
				</li>
				<li>
					<label>
						<input type="checkbox" name="exportItems[]" value="themeOptions" <?php echo( in_array( get_option('template'), array('Divi', 'Extra') ) ? 'checked' : 'disabled' ); ?>>
						<?php esc_html_e('Theme options', 'wp-layouts-td');?>
					</label>
					
					<p class="ags-layouts-site-export-instructions"><?php esc_html_e('Only Divi and Extra are supported for theme options export in this beta version of the site exporter. If you would like us to support a different theme, please let us know!', 'wp-layouts-td');?></p>
				</li>
                <li>
                    <label>
                        <input type="checkbox" name="exportItems[]" value="widgets" checked>
                        <?php esc_html_e('Widgets', 'wp-layouts-td');?>
                    </label>
                </li>
				<li>
					<label>
						<?php esc_html_e('Plugin options', 'wp-layouts-td');?>
					</label>
					
					<ul>
						<li>
							<label>
								<input type="checkbox" name="exportItems[]" value="pluginOptions.TheEventsCalendar" <?php echo( get_option('tribe_events_calendar_options') ? 'checked' : 'disabled' ); ?>>
								<?php esc_html_e('The Events Calendar', 'wp-layouts-td');?>
							</label>
						</li>
					</ul>
					
					<p class="ags-layouts-site-export-instructions"><?php esc_html_e('We are planning to add support for more plugins as we improve this beta version of the site exporter. Please let us know if you need support for a particular plugin!', 'wp-layouts-td');?></p>
				</li>
				<?php if ( AGSLayoutsSiteExportCalderaForms::isSupported() ) { ?>
				<li>
					<label>
                        Caldera Forms
					</label>
					
					<ul>
						<?php foreach ( AGSLayoutsSiteExportCalderaForms::getForms() as $formId => $formName) { ?>
						<li>
							<label>
								<input type="checkbox" name="exportItems[]" value="calderaForms.<?php echo esc_attr($formId); ?>" checked>
								<?php echo esc_html($formName); ?>
							</label>
						</li>
						<?php } ?>
					</ul>
					
				</li>
				
				<?php } ?>
			</ul>
			
			<div id="ags-layouts-site-export-status"></div>
			
			<?php wp_nonce_field('ags-layouts-site-export-ajax', 'ags_layouts_nonce'); ?>
		</form>
	</div>
</div>


<script>
jQuery(document).ready(function($) {
	var $form = $('#ags-layouts-site-export form');
	var $status = $('#ags-layouts-site-export-status');
	
	// wp-layouts\includes\site-export-packager-page.php
	var errorHandler = function() {
		console.log( 'Site export error' + (arguments.length > 1 ? ': ' + arguments[1] : '') );
		if (arguments.length > 2) {
			console.log(arguments[2]);
		}

		// jh-todo
		ags_layouts_message_dialog(
			'Error',
			'Something went wrong during the export.',
			'O'
		);
	};

	var dialogOptions = {
		title: 'WP Layouts Site Exporter',
		container: $form.parent(),
		content: $form,
		pageName: 'site-export',
		firstButtonClass: 'aspengrove-btn-secondary',
		buttons: {
			'Export Site': function() {
		
				var layoutName = $('#ags-layouts-site-export-name').val();
				var layoutData = {
					contents: '',
					extraData: {}
				};
				var tasks = [];
				var selectedTasks = {};
				var screenshotData = -1;
				
				$('#ags-layouts-site-export-items input:checked').each(function() {
					var checkedItem = $(this).val().split('.');
					if (checkedItem.length == 1) {
						selectedTasks[ checkedItem[0] ] = true;
					} else if (selectedTasks[ checkedItem[0] ] && selectedTasks[ checkedItem[0] ].length) {
						selectedTasks[ checkedItem[0] ].push(checkedItem[1]);
					} else {
						selectedTasks[ checkedItem[0] ] = [ checkedItem[1] ];
					}
				});
				
				tasks.push([
					'Exporting content...',
					function(cb) {
						/*
								This function contains code adapted from fast-xml-parser-3.16.0\README.md
								
								MIT License

								Copyright (c) 2017 Amit Kumar Gupta

								Permission is hereby granted, free of charge, to any person obtaining a copy
								of this software and associated documentation files (the "Software"), to deal
								in the Software without restriction, including without limitation the rights
								to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
								copies of the Software, and to permit persons to whom the Software is
								furnished to do so, subject to the following conditions:

								If you use this library in a public repository then you give us the right to mention your company name and logo in user's list without further permission required, but you can request them to be taken down within 30 days. 

								The above copyright notice and this permission notice shall be included in all
								copies or substantial portions of the Software.

								THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
								IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
								FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
								AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
								LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
								OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
								SOFTWARE.
						*/
					
						$.get(
							'export.php?agslayouts=1',
							{
								download: true,
							},
							function(responseString) {
								
								var result = agslayouts_xml_parser.validate(responseString, {
								
								});
								if (result !== true) console.log(result.err);
								var response = agslayouts_xml_parser.parse(responseString, {
									ignoreAttributes: false,
									cdataTagName: "__cdata",
								});
							
								var excludePostTypes = $('#ags-layouts-site-export [name="excludePostTypes"]:first').val();
								if (excludePostTypes) {
									excludePostTypes = excludePostTypes.split(',');
									for (var i = 0; i < excludePostTypes.length; ++i) {
										excludePostTypes[i] = excludePostTypes[i].trim();
									}
								} else {
									excludePostTypes = [];
								}
								excludePostTypes.push('custom_css');
								
								var validItems = [];
								for (var i = 0; i < response.rss.channel.item.length; ++i) {
									if ( excludePostTypes.indexOf( response.rss.channel.item[i]['wp:post_type'].__cdata ? response.rss.channel.item[i]['wp:post_type'].__cdata : response.rss.channel.item[i]['wp:post_type'] ) == -1) {
										validItems.push(response.rss.channel.item[i]);
									}
								}
								response.rss.channel.item = validItems;
								
							
								var excludeTaxonomies = $('#ags-layouts-site-export [name="excludeTaxonomies"]:first').val();
								if (excludeTaxonomies) {
									excludeTaxonomies = excludeTaxonomies.split(',');
									for (var i = 0; i < excludeTaxonomies.length; ++i) {
										excludeTaxonomies[i] = excludeTaxonomies[i].trim();
									}
									
									var validTerms = [];
									for (var i = 0; i < response.rss.channel['wp:term'].length; ++i) {
										if ( excludeTaxonomies.indexOf( response.rss.channel['wp:term'][i]['wp:term_taxonomy'].__cdata ? response.rss.channel['wp:term'][i]['wp:term_taxonomy'].__cdata : response.rss.channel['wp:term'][i]['wp:term_taxonomy'] ) == -1) {
											validTerms.push(response.rss.channel['wp:term'][i]);
										}
									}
									response.rss.channel['wp:term'] = validTerms;
									
								}
								
								var j2xParser = new agslayouts_xml_parser.j2xParser({
									ignoreAttributes: false,
									cdataTagName: "__cdata",
									format: true,
									indentBy: '',
								});
								layoutData.contents = j2xParser.parse(response);
								layoutData.contents = layoutData.contents
														.replace(/\n\<!\[CDATA\[/g, '<![CDATA[')
														.replace(/\]\]\>\n/g, ']]>');
								
								cb();
							}, 'text'
						)
						.fail(errorHandler);
					}
				]);
				
				if ( selectedTasks.widgets ) {
					tasks.push([
						'Exporting widgets...',
						function(cb) {
							$.get(
								ajaxurl,
								{
									action: 'ags_layouts_get_widgets_export',
									ags_layouts_nonce: $('#ags-layouts-site-export #ags_layouts_nonce').val(),
								},
								function(response) {
									layoutData.extraData.widgets = response;
									cb();
								},
								'text'
							)
							.fail(errorHandler);
						}
					]);
				}
				
				if ( selectedTasks.themeOptions || selectedTasks.pluginOptions ) {
					tasks.push([
						'Exporting theme and/or plugin options...',
						function(cb) {
							$.post(
								ajaxurl,
								{
									action: 'ags_layouts_get_theme_plugin_options_export',
									ags_layouts_nonce: $('#ags-layouts-site-export #ags_layouts_nonce').val(),
									themeOptions: selectedTasks.themeOptions ? true : false,
									pluginOptions: selectedTasks.pluginOptions ? selectedTasks.pluginOptions : null
								},
								function(response) {
									layoutData.extraData.agsxto = response;
									cb();
								},
								'text'
							)
							.fail(errorHandler);
						}
					]);
				}
				
				if ( selectedTasks.calderaForms ) {
					tasks.push([
						'Exporting Caldera Forms...',
						function(cb) {
							$.post(
								ajaxurl,
								{
									action: 'ags_layouts_get_caldera_forms_export',
									ags_layouts_nonce: $('#ags-layouts-site-export #ags_layouts_nonce').val(),
									formIds: selectedTasks.calderaForms
								},
								function(response) {
									layoutData.extraData.caldera_forms = response;
									cb();
								},
								'text'
							)
							.fail(errorHandler);
						}
					]);
				}
				
				tasks.push([
						'Exporting menu assignments...',
						function(cb) {
							$.post(
								ajaxurl,
								{
									action: 'ags_layouts_get_menu_assignments_export',
									ags_layouts_nonce: $('#ags-layouts-site-export #ags_layouts_nonce').val(),
								},
								function(response) {
									layoutData.extraData.config = {};
									
									if (response) {
										layoutData.extraData.config.menus = response;
									}
									
									// below string from my WordPress export XML file, modified the type
									if (layoutData.contents.indexOf('<wp:post_type><![CDATA[et_template]]></wp:post_type>') != -1) {
										layoutData.extraData.config.hasThemeBuilderTemplates = true;
									}
									
									layoutData.extraData.config = JSON.stringify(layoutData.extraData.config);
									
									cb();
								},
								'json'
							)
							.fail(errorHandler);
						}
					]);
				
				tasks.push([
					'Processing image...',
					function(cb) {
						
						// The following code is copied from https://developer.mozilla.org/en-US/docs/Web/API/FileReader/readAsDataURL and modified. Original code is in the public domain.
						  const file = $('#ags-layouts-site-export-image')[0].files[0];
						  const reader = new FileReader();

						  reader.addEventListener("load", function () {
							screenshotData = reader.result;
							cb();
						  }, false);

						  if (file) {
							reader.readAsDataURL(file);
						  } else {
							cb();
						  }
						// End code is copied from https://developer.mozilla.org/en-US/docs/Web/API/FileReader/readAsDataURL and modified.
						
						
					}
				]);
				
				tasks.push([
					'Preparing to upload...',
					function() {
						
						ags_layouts_export_job_ui(
							layoutData.contents,
							screenshotData,
							'site-importer',
							layoutName,
							dialog,
							$form,
							dialog.setButtons,
							dialog.close,
							layoutData.extraData
						);
					}
				]);
				
				function runTask() {
					if (tasks.length) {
						var task = tasks.shift();
						$status.text(task[0]);
						task[1](runTask);
					}
				}
				runTask();
				
				return false;
			}
			
		}
	};
	
	dialog = agsLayoutsDialog.create(dialogOptions);
	
	
});
</script>